//
// C++ Implementation: analyze
//
// Description:
//
//
// Author: Jiří Nymš <jirka@nyms.cz>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <iomanip>
#include <sstream>
#include "analyze.h"
using namespace std;

#define HEIGHT 600

//procedura shoda s tou v batmonitord.cpp
void split_string(const string& original, string& s1, string& s2, const char splitter)
{
  unsigned int  position = original.find(splitter);
  s2 = original.substr(position + 1, original.size());
  s1 = original.substr(0, original.find(splitter));
}

bool from_string(unsigned int& n, const std::string& s)
{
  istringstream iss(s);
  return !(iss >> dec >> n).fail();
}
list<string> get_values(const string& original, const char splitter)
{
  list<string> l=list<string>();
  unsigned int position=0;
  string s1, s2=original;
  while (position<s2.size()) {
    position=s2.find(splitter);
    s1 = s2.substr(0, s2.find(splitter));
    s2 = s2.substr(position + 1, s2.size());
    l.push_back(s1);
  }
  l.push_back(s2);
  return l;
}

CState::CState()
{
  present_rate=0;
  remaining_capacity=0;
  num=1;
  freq=0;
}
CState::CState(int present_rate, int remaining_capacity, int num, int freq)
{
  this->present_rate=present_rate;
  this->num=num;
  this->freq=freq;
  this->remaining_capacity=remaining_capacity;
}
CState::~CState() {}

bool CState::operator==(const CState& c) const
{
  return (present_rate==c.present_rate && freq==c.freq && remaining_capacity==c.remaining_capacity);
}
bool CState::operator>(const CState& c) const
{
  return remaining_capacity > c.remaining_capacity;
}

string int2str(int i)
{
  string s;
  ostringstream oss(s);
  oss << i;
  return oss.str();
}

//exportuje data do svg "obrazku"
void SVGExport::init(int width, int height)
{
  //cout << "init(width, height)" << endl;
  //svg.clear();

  svg.push_back("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>");
  svg.push_back("<svg xmlns=\"http://www.w3.org/2000/svg\" width=\""+int2str(width)+"px\" height=\""+int2str(height+15)+"px\">");
  svg.push_back("  <polyline points=\"0,"+int2str(height)+" "+int2str(width)+","+int2str(height)+" "+int2str(width)+",0\" stroke=\"black\" fill=\"none\" />\n");
  for (int j=1;j<6;j++) {
    svg.push_back("  <line x1=\"0\" y1=\""+int2str(j)+"00\" x2=\""+int2str(width)+"\" y2=\""+int2str(j)+"00\" stroke=\"black\" />");
  }
  for (int j=100;j<width;j+=100) {

    svg.push_back("  <line x1=\""+int2str(j)+"\" y1=\"0\" x2=\""+int2str(j)+"\"  y2=\""+int2str(height)+"\" stroke=\"black\" />");
    svg.push_back("<text x=\""+int2str(j+2)+"\" y=\"615\" font-family=\"sans-serif\" font-size=\"11\" fill=\"black\" >"+int2str(j)+"0 s</text>");
  }
  svg.push_back("  <polyline points=\"");
}

SVGExport::SVGExport(int width, int height)
{
  init(width, height);
}
SVGExport::SVGExport()
{}

void SVGExport::add_values(const string& values)
{
  svg.back()+=" "+values;
}

void SVGExport::export_to_file(const string& filename) const
{
  
  ofstream of(filename.c_str());
  for (list<string>::const_iterator it=svg.begin(); it!=svg.end(); it++) {
    of << *it << endl;
	cout << *it << endl;
  }
  of << "</svg>"; 
  of.close();
}

void SVGExport::finish()
{
  svg.push_back("\"    stroke=\"red\" fill=\"none\" />\n");
}

void SVGExport::add_other(const string& str)
{
  svg.push_back(str);
}

ostream& operator<<(ostream& os, const SVGExport& svg)
{
  //budeme predpokladat, ze SVG je spravne vygenerovany
  for (list<string>::const_iterator it=svg.svg.begin();it!=svg.svg.end();it++) {
    os<<*it;
  }
  os << "</svg>" << endl;
  return os;
}


BatteryLife::BatteryLife()
{
  //cout << "BatteryLife()" << endl;
  items=list<CState>();
  total_time=0;
}

void BatteryLife::setBattery(const TBattery* battery)
{
  //info=battery;
  design_capacity=battery->design_capacity;
  name=battery->name;
  design_voltage=battery->design_voltage;
  last_full=battery->last_full;
}

void BatteryLife::add(const CState& item)
{
  //cout << "BatteryLife::add()" << endl;
//pokud má poslední položka stejný obsah, tak se zvýší délka intervalu
  total_time+=30;
  if (!items.empty()) {//kontrola shodnosti posledni polozky
    if (item==items.back()) {
      items.back().num++;
      return;
    }
  }
  items.push_back(item);
  if (total_time==1800) {
	//first_
	//angle=atan();
  }
}

const CState& BatteryLife::getlast(void) const
{
  return items.back();
}

unsigned int BatteryLife::getTotalTime() const
{
  return total_time;
}

const QByteArray& BatteryLife::get_freq_data() const
{
  return qf;
}
const QByteArray& BatteryLife::get_power_data() const
{
  return qp;
}
const QByteArray& BatteryLife::get_capacity_data() const
{
  return qc;
}


int get_position(int cur, int max)
{
  return (600 - (int) cur*HEIGHT/max);
}

void BatteryLife::create_freq_svg()
{
  //cout << "create_freq_svg()" << endl;

#define STEP_WIDTH 3
#define MAX_FREQ 3000
  stringstream sf;
  int width=(int)((total_time/30)*STEP_WIDTH);
  freq.init(width, HEIGHT);
  //neni to nic hezkyho, ale dejme tomu, ze max. freq budou 3GHz=>600px, CPU stejne jdou cestou vice jader, casem to budu muset poresit jinak
  int n=0;
  unsigned int tmp=0, i=0;
  string a, b;
  for (list<CState>::const_iterator it=items.begin();it!=items.end();it++) {
    n+=it->num;
    if (it->freq!=tmp) {
      a=int2str(get_position(it->freq, MAX_FREQ));
      b=int2str(n*STEP_WIDTH);
      if (i!=1 && tmp!=0)

        freq.add_values(int2str((n-1)*STEP_WIDTH)+","+int2str(get_position(tmp,MAX_FREQ)));
      freq.add_values(b+","+a);
      i=1;
    } else
      i++;
    tmp=it->freq;
  }
  //if () svg.add_values(int2str(n*STEP_WIDTH)+","+int2str());
  freq.add_values(int2str((int)total_time*STEP_WIDTH/30)+","+int2str(get_position(tmp,MAX_FREQ)));
  freq.finish();
  for (int j=0;j<6;j++) {
    freq.add_other("<text x=\"2\" y=\""+int2str(600-j*100-2)+"\" font-family=\"sans-serif\" font-size=\"10\" fill=\"black\" >"+int2str(j*600*5/6)+" MHz</text>");
  }
  sf  << freq;
  qf=sf.str().c_str();
}

void BatteryLife::create_power_svg()
{
  //cout << "create_power_svg()" << endl;
  stringstream sp;
  int width=(int)((total_time/30)*STEP_WIDTH);
  power.init(width, HEIGHT); //pokud maji adaptery k notebookum max. odber 65W, tak se da ocekavat, ze notebooky nebudou zas tak moc "zrat", tzn max. spotrebu bych odhadl na 50W
#define MAX_POWER 50000
  //duvody jsou zhruba takovy, ze stejne senzory nedokazi spravne zobrazit ani design_capacity

  int n=0;
  unsigned int tmp=0, i=0;
  string a, b;

  for (list<CState>::const_iterator it=items.begin();it!=items.end();it++) {
    n+=it->num;
    if (it->present_rate!=tmp) {
      a=int2str(get_position(it->present_rate, MAX_POWER)); //600-(int)it->present_rate*HEIGHT/MAX_POWER);
      b=int2str(n*STEP_WIDTH);
      if (i!=1 && tmp!=0)
        power.add_values(int2str((n-1)*STEP_WIDTH)+","+int2str(get_position(tmp, MAX_POWER)));
      power.add_values(b+","+a);
      i=1;
    } else
      i++;
    tmp=it->present_rate;
  }
  power.add_values(int2str((int)total_time*STEP_WIDTH/30)+","+int2str(get_position(tmp, MAX_POWER)));
  power.finish();
  for (int j=0;j<6;j++) {
	
    power.add_other("<text x=\"2\" y=\""+int2str(600-j*100-2)+"\" font-family=\"sans-serif\" font-size=\"10\" fill=\"black\" >"+int2str(j*14*600)+" W</text>");
  }

  sp  << power;
  qp=sp.str().c_str();

}

void BatteryLife::export_freq_svg(const char* filename) const
{
  if (strcmp(filename,"")) {
    freq.export_to_file(filename);
    return;
  }
}

void BatteryLife::export_power_svg(const char* filename) const
{
  if (strcmp(filename,"")) {
    power.export_to_file(filename);
    return;
  }
}

void BatteryLife::export_capacity_svg(const char* filename) const
{
  if (strcmp(filename,"")) {
    capacity.export_to_file(filename);
    return;
  }
}
void BatteryLife::create_capacity_svg()
{
  //cout << "create_capacity_svg()" << endl;

  stringstream s;
#define MAX_CAPACITY 75000
  int width = (int)((total_time/30)*STEP_WIDTH);
  //neni to nic hezkyho, ale dejme tomu, ze max. spotreba bude 50W=>600px(planuju to stejne pro notebooky, ktery teto hodnoty dosahovat nebudou)
  int n=0;
  unsigned int tmp=0, i=0;
  string a, b;

  capacity.init(width, HEIGHT);
  //cout << "initialed" << endl;
  for (list<CState>::const_iterator it=items.begin();it!=items.end();it++) {
    n+=it->num;
    if (it->remaining_capacity!=tmp) {
      a=int2str(get_position(it->remaining_capacity,MAX_CAPACITY));
      b=int2str(n*STEP_WIDTH);

      if (i!=1 && tmp!=0)
        capacity.add_values(int2str((n-1)*STEP_WIDTH)+","+int2str(get_position(tmp,MAX_CAPACITY)));
      capacity.add_values(b+","+a);

      i=1;
    } else
      i++;
    tmp=it->remaining_capacity;
  }
  capacity.add_values(int2str((int)total_time*STEP_WIDTH/30)+","+int2str(get_position(tmp,MAX_CAPACITY)));
  capacity.finish();
  //cout << info->design_capacity << endl;
  capacity.add_other("  <line x1=\"0\" y1=\""+int2str(get_position(design_capacity,MAX_CAPACITY))+"\" x2=\""+int2str(width)+"\" y2=\""+int2str(get_position(design_capacity,MAX_CAPACITY))+"\" stroke=\"green\" />");

  for (int j=0;j<6;j++) {
    capacity.add_other("<text x=\"2\" y=\""+int2str(600-j*100-2)+"\" font-family=\"sans-serif\" font-size=\"10\" fill=\"black\" >"+int2str(j*11000*100/MAX_CAPACITY)+" Wh</text>");
  }
  s  << capacity;
  //cout << capacity << endl;
  qc=s.str().c_str();
}

Battery::Battery()
{
  init=false;
}

Battery::Battery(const TBattery& bat)
{
  history=list<BatteryLife>();
  info=bat;
  init=true;
}
void Battery::add_history(BatteryLife& life)
{
  life.setBattery(&this->info);
  history.push_back(life);
}
void Battery::append_state(CState& state)
{
  if (history.empty() || state>history.back().getlast()) {
    BatteryLife bl=BatteryLife();
    bl.setBattery(&info);
    history.push_back(bl);
  }
  list<BatteryLife>::iterator it= --history.end();
  it->add(state);
}
//tady ještě nevím, jaký typ použiju, ale spíš referenci
const TBattery& Battery::getinfo(void) const
{
  return info;
}

string Battery::getname() const
{
  return info.name;
}
unsigned int Battery::getvoltage()
{
  return info.design_voltage;
}
bool Battery::operator==(const Battery& b) const
{

  return (info.design_capacity==b.getinfo().design_capacity &&
          info.design_voltage==b.getinfo().design_voltage &&
          info.last_full==b.getinfo().last_full);
}

vector<Battery>::iterator find_battery(const string& name, vector<Battery>& batteries)
{
  for (vector<Battery>::iterator it=batteries.begin(); it!=batteries.end(); it++) {
    if (it->getname().compare(name)==0) {
      return it;
    }
  }
  return batteries.end();
}

//zpracovani radku s flagem info
void LogReader::info(const string& s)
{
  string s1, s2, s3;
  TBattery info, tmp;

  list<string> values=get_values(s, ';');
  list<string>::const_iterator it=values.begin();
  info.name=*it++;
//nejrychlejsi vyreseni problemu se zpetnou kompatibilitou starsi verze logu

  split_string(*it++, s1, s3, ' ');
  from_string(tmp.design_capacity, s1);
  split_string(*it++, s1, s3, ' ');
  from_string(tmp.last_full, s1);
  split_string(*it++, s1, s2, ' ');
  from_string(info.design_voltage, s1);

  if (s2.compare("mWh")==0) { //ale neresi to, kdyz se uvadeji spatny jednotky...
    info.design_capacity=tmp.design_capacity;
    info.last_full=tmp.last_full;
  } else {//v mAh, zaokrouhlovací chybu zanedbáme
    info.design_capacity=(unsigned int)(tmp.design_capacity*info.design_voltage/1000);
    info.last_full=(unsigned int)(tmp.last_full*info.design_voltage/1000);
  }
  //ma to fungovat zhruba tak, ze pokud se nenajde stejna baterka, tak se na to vykasle
  bool found=false;

  Battery b=Battery(info);
  for (vector<Battery>::const_iterator it=batteries->begin(); it!=batteries->end(); it++) {
    if (!b.getname().compare(it->getname())) {
      found=true;
    }
  }
  if (!found) {
    batteries->push_back(b);
    //cout << "dalsi baterka"<< endl;
  }
}

void LogReader::load(ifstream& in)
{
  string line, s1, s2, s3, ll;
  while (getline(in, line)) {
    //stringy uz jsou stripnuty, neni potreba je stripnout znova
    split_string(line, s1, s2, ';');

    //nejdriv se ujisti, jestli prvni polozka neni "info"
    if (s1.compare("info")==0) {
      info(s2);
    } else
      if (!batteries->empty()) {

        //bezny stav
        list<string> rawstate=get_values(line, ';');
        list<string>::const_iterator it=rawstate.begin();
        CState state=CState(), tmp=CState();
        unsigned int voltage;
        string name=*it++;
        int n=0;
        vector<Battery>::iterator it2;


        //???? nebylo by lepsi pouzit proceduru na hledani???
        //nesahame do kodu, ktery funguje
        for (vector<Battery>::const_iterator it3=batteries->begin(); it3!=batteries->end(); it3++) {
          if (!name.compare(it3->getname())) {
            n = it3-batteries->begin();
          }
        }
		//zpetna kompatibilita
        while (!it->compare("yes") || !it->compare("ok") || !it->compare("discharging") || !it->compare("discharging") || !it->compare("critical")) {
          *it++;
        }
        it2=batteries->begin()+n;
        //spotreba nebo proud
        split_string(*it++, s1, s2, ' ');
        from_string(tmp.present_rate, s1); //pokud by to bylo v mA, musel bych to prevest a ten udaj jeste neznam

        //zbyvajici kapacita
        split_string(*it++, s1, s3, ' ');
        from_string(tmp.remaining_capacity, s1);

        //napajeni
        split_string(*it++, s1, line, ' ');//mV
        from_string(voltage, s1);
        //frekvence
        from_string(state.freq, *it);

        if (s2.compare("mA")) {
          state.present_rate=tmp.present_rate;
        } else {
          state.present_rate=(unsigned int) tmp.present_rate*voltage/1000;
        }
        if (s3.compare("mAh")) {
          state.remaining_capacity=tmp.remaining_capacity;
        } else {
          state.remaining_capacity=(unsigned int) it2->getvoltage() * tmp.remaining_capacity/1000;
        }
        it2->append_state(state);
      }
  }
}
LogReader::LogReader(const char*  filename)
{
  batteries = new vector<Battery>();
  ifstream in(filename);
  if (in.is_open()) {
    load(in);
    in.close();
  }
}

LogReader::~LogReader()
{
  delete batteries;
}

vector<Battery>* LogReader::getbatteries()
{
  return batteries;
}

list<BatteryLife>& Battery::gethistory()
{
  return history;
}

unsigned int BatteryLife::get_last_full() const
{
  return last_full;
}

unsigned int BatteryLife::get_design_capacity() const{
  return design_capacity;
}

unsigned int BatteryLife::get_design_voltage() const{
  return design_voltage;
}

string BatteryLife::getname() const{
  return name;
}

