//
// C++ Header: analyze
//
// Description:
//
//
// Author: Jiří Nymš <jirka@nyms.cz>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
#ifndef analyze_h__1f41e4334eb7dd1759d3e6b6ca932a52edfad974__
#define analyze_h__1f41e4334eb7dd1759d3e6b6ca932a52edfad974__
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <list>
#include <QByteArray>

//procedura shoda s tou v batmonitord.cpp
void split_string(const std::string& original, std::string& s1, std::string& s2, const char splitter=':');
bool from_string(unsigned int& n, const std::string& s);
std::list<std::string> get_values(const std::string& original, const char splitter=':');

class CState
{
  public:
    unsigned int present_rate; //aktuální spotřeba(mW)
    unsigned int remaining_capacity; //zbývající kapacita(mWh)
    unsigned int num; //počet stejných záznamů-pro vyjádření skutečné výdrže, každý záznamy se vytváří vždy po 30s
    unsigned int freq; //frekvence CPU
    CState();
    CState(int present_rate, int remaining_capacity, int n, int freq);
    ~CState();
    bool operator==(const CState& c) const;
    bool operator>(const CState& c) const;
};

struct TBattery {
  std::string name;
  unsigned int design_capacity; //mWh, doufám, že (design voltage*design capacity(mAh)) == design capacity(mWh)
  unsigned int last_full; //mWh
  unsigned int design_voltage; //mV
};

std::string int2str(int i);

//exportuje data do svg "obrazku"
class SVGExport
{
    std::list<std::string> svg;
  public:
    SVGExport();
    SVGExport(int width, int height);
    void init(int width, int height);
    void add_values(const std::string& values);
    void add_other(const std::string& values);
    void finish();
    void export_to_file(const std::string& filename) const;
    friend std::ostream& operator<<(std::ostream& os, const SVGExport& svg);
};
class BatteryLife
{
    unsigned int total_time, design_voltage, design_capacity, last_full;
    std::string name; //vubec se mi to nelibi, ale s TBattery mi to nefungovalo
    std::list<CState> items;
    SVGExport power, freq, capacity;
    QByteArray qp, qf, qc;
  public:
    BatteryLife();
    void add(const CState& item);
    void setBattery(const TBattery* battery);
    unsigned int getTotalTime() const;
	std::string getname() const;
	unsigned int get_design_capacity() const;
	unsigned int get_design_voltage() const;
	unsigned int get_last_full() const;
    const CState& getlast() const;
    void create_freq_svg();
    void create_power_svg();
    void create_capacity_svg();
    void export_freq_svg(const char* filename="") const;
    void export_power_svg(const char* filename="") const;
    void export_capacity_svg(const char* filename="") const;
    const QByteArray& get_freq_data() const;
    const QByteArray& get_power_data() const;
    const QByteArray& get_capacity_data() const;
};

class Battery
{
    unsigned int longest_time;
    std::list<BatteryLife> history;
    TBattery info;
    bool init;
  public:
    //nevidim to tu rad, ale co se da delat
    Battery();
    Battery(const TBattery& bat);
    void add_history(BatteryLife& life);
    void append_state(CState& state);
    //tady ještě nevím, jaký typ použiju, ale spíš referenci
    const TBattery& getinfo(void) const;
    std::string getname() const;
    unsigned int getvoltage();
    bool operator==(const Battery& b) const;
    std::list<BatteryLife>& gethistory();
};

std::vector<Battery>::iterator find_battery(const std::string& name, std::vector<Battery>& batteries);

class LogReader
{
    std::string name;
    std::vector<Battery> * batteries;
    //zpracovani radku s flagem info
    void info(const std::string& s);
    void load(std::ifstream& in);
  public:
    LogReader(const char*  filename);
    ~LogReader();
    std::vector<Battery>* getbatteries(void);
};
#endif
