//
// C++ Implementation: mywindow
//
// Description:
//
//
// Author: Jiří Nymš <jirka@nyms.cz>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <QtGui>
#include <QFileDialog>
#include "mywindow.h"
#include <stdlib.h>
#include <sstream>
#include <QList>
#include <time.h>
#include <QSvgRenderer>
using namespace std;

SvgPlot::SvgPlot(QWidget *parent): QSvgWidget(parent)
{
  p=parent;
}

SvgPlot::~SvgPlot()
{
}

QSize SvgPlot::sizeHint() const
{

  int w,h;
  w=renderer()->defaultSize().width();
  h=renderer()->defaultSize().height();
  if (p && w>p->width()) {
    double ratio=h*1000/w;
    w=p->width();
    h=(int) w*ratio/1000;

  }

  return QSize(w,h);
}


MyWindow::MyWindow(QMainWindow *parent) : QMainWindow(parent)
{
  setupUi(this);
  closed=false;
  actionRestore=new QAction(tr("&Restore"), this);
  label->setText("");
  label_2->setText("");
  label_3->setText("");
  label_4->setText("");
  label_5->setText("");
  label_6->setText("");
  opened=false;

  ptimer=new QTimer(this);
  ptimer->start(30000);
  QByteArray filename(getenv("HOME"));
  filename+="/";
  filename+=PIPE_FILE;
  pipe=new CPipe(filename.data());

  createTrayIcon();
  power_graph=new SvgPlot(tab_2);
  freq_graph=new SvgPlot(tab_3);
  capacity_graph=new SvgPlot(tab_4);

  connect(actionKonec, SIGNAL(triggered()), qApp, SLOT(quit()));
  connect(actionRestore, SIGNAL(triggered()), this, SLOT(showNormal()));
  connect(ptimer, SIGNAL(timeout()), this, SLOT(read_pipe()));
  connect(tray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
          this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}

MyWindow::~MyWindow()
{
  //qDebug() << "~MyWindow()";
  if (opened)
    delete lr;
  delete pipe;
  delete power_graph;
  delete freq_graph;
  delete capacity_graph;
  delete tray;
  delete ptimer;
  delete trayMenu;
  delete actionRestore;

}

void MyWindow::on_actionOpen_triggered()
{
  QString fileName = QFileDialog::getOpenFileName(this,
                     tr("Log file"), getenv("HOME"), tr("Log files (*.log)"));
  if (!fileName.isEmpty()) {
    if (opened) {
      listWidget->clear();
      delete lr;
    }
    opened=true;
    lr=new LogReader(fileName.toAscii());
    load_data();
  }
}

void MyWindow::on_actionExport_triggered()
{
  //qDebug() << tabWidget->currentIndex();
  if (opened) {
    QString fileName = QFileDialog::getSaveFileName(this,
                       tr("Log file"), getenv("HOME"), tr("SVG files (*.svg)"));
    list<BatteryLife>::const_iterator cur = global_history[listWidget->currentRow()];
    switch (tabWidget->currentIndex()) {
      case 1:
        cur->export_power_svg(fileName.toAscii());
        break;
      case 2:
        cur->export_freq_svg(fileName.toAscii());
        break;
      case 3:
        cur->export_capacity_svg(fileName.toAscii());
        break;
      default:
        QMessageBox::warning(this, tr("Battery Analyzer"), tr("Image wasn't selected."), QMessageBox::Ok, QMessageBox::Ok);
    }
  } else
    QMessageBox::warning(this, tr("Battery Analyzer"),
                         tr("Log file wasn't loaded."),
                         QMessageBox::Ok,  QMessageBox::Ok);
}

void MyWindow::load_data()
{
  //qDebug() << "load_data()";
  vector<Battery> *batteries = lr->getbatteries();
  global_history.clear();
  for (vector<Battery>::iterator it=batteries->begin(); it!=batteries->end(); it++) {
    prefix=it->getname().c_str();
    for (list<BatteryLife>::iterator it2=it->gethistory().begin();it2!=it->gethistory().end(); it2++) {

      it2->create_capacity_svg();
      it2->create_freq_svg();
      it2->create_power_svg();

      global_history.push_back(it2);
      stringstream s;
      s<<it2->getTotalTime();
      listWidget->addItem(prefix+":"+s.str().c_str()+tr(" seconds"));
    }
    listWidget->setCurrentRow(0);
    select_battery();
  }

  connect(listWidget, SIGNAL(itemSelectionChanged()), this, SLOT(select_battery()));
}

const char* maketime(int seconds)
{

  int hours=seconds/3600;
  int minutes=(seconds-hours*3600)/60;
  int s=seconds-hours*3600-minutes*60;
  return (int2str(hours)+":"+int2str(minutes)+":"+int2str(s)).c_str();
}

void MyWindow::select_battery()
{
  //qDebug() << "select_battery()";
  list<BatteryLife>::const_iterator cur = global_history[listWidget->currentRow()];
  string selected=cur->getname();
  vector<list<BatteryLife>::const_iterator>::const_iterator i;
  unsigned int longest=0;
  if (!cur->getname().compare(selected)) {
    for (i=global_history.begin();i<global_history.end();i++) {
      if ((*i)->getTotalTime()>longest) {
        longest=(*i)->getTotalTime();
      }
    }

  }

  label->setText(tr("Battery ")+cur->getname().data());
  label_2->setText(tr("Design capacity: ")+int2str(cur->get_design_capacity()).c_str()+" mW");
  label_3->setText(tr("Last full capacity: ")+int2str(cur->get_last_full()).c_str()+" mW");
  label_4->setText(tr("Design voltage: ")+int2str(cur->get_design_voltage()).c_str()+" mV");


  label_5->setText(tr("The longest time: ")+maketime(longest));


  power_graph->load(cur->get_power_data());
  power_graph->setFixedSize(power_graph->sizeHint());
  freq_graph->load(cur->get_freq_data());
  freq_graph->setFixedSize(freq_graph->sizeHint());
  capacity_graph->load(cur->get_capacity_data());
  capacity_graph->setFixedSize(capacity_graph->sizeHint());

}

void MyWindow::read_pipe()
{
  open_pipe(0);
}



void MyWindow::open_pipe(int n)
{
  QString msg="";
  if (pipe->is_open()) {
    QByteArray line;
    line = pipe->pread();
    if (line=="") {
      msg=tr("Daemon doesn't run.");
      label_6->setText(msg);
      tray->setToolTip(msg);
      return;
    }
    QList<QByteArray> bats=line.split(';');
    for (int i = 0; i < bats.size(); i++) {
      QByteArray bat=bats.takeAt(i);

      time_t since;
      if (bat.split(':').size()>1)
        since = bat.split(':').takeAt(1).toLong();
      else
        since=-1;

      if (since>0 && opened) {

        list<BatteryLife>::const_iterator cur = global_history[listWidget->currentRow()];
        int duration=time(NULL)-since;
        msg=tr("Elapsed time: ")+maketime(duration);
        msg+=tr(" s, expected time: ")+maketime(cur->getTotalTime()-duration)+" s";
      } else
        if (!opened && since>0) {
          msg=tr("Battery history isn't selected.");
        } else
          if (since==0) {
            msg=tr("AC adapter is plugged.");
          } else {
            msg=tr("Unexpected problem in comunication with daemon.");
          }
    }
    label_6->setText(msg);
    tray->setToolTip(msg);
  } else
    if (n==0) {
      pipe->reopen();
      open_pipe(++n);
    } else {
      label_6->setText("Can't open named pipe to daemon.");
    }
}
void MyWindow::closeEvent(QCloseEvent *event)
{
  tray->setVisible(true);
  if (!closed)
    QMessageBox::information(this, tr("Systray"),
                             tr("This program will keep running in the "
                                "system tray. To close the program, "
                                "choose <b>Quit</b> in the context menu "
                                "of the system tray entry."));
  closed=true;
  hide();
  event->ignore();
}

void MyWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
  switch (reason) {
    case QSystemTrayIcon::Trigger:
    case QSystemTrayIcon::DoubleClick:
      showNormal();
      break;
    default:
      ;
  }
}

void MyWindow::createTrayIcon()
{
  trayMenu = new QMenu(this);
  trayMenu->addAction(actionRestore);
  trayMenu->addSeparator();
  trayMenu->addAction(actionKonec);

  tray = new QSystemTrayIcon(this);
  tray->setContextMenu(trayMenu);
  QIcon icon;
  icon.addPixmap(QPixmap(QString::fromUtf8(":/icons/battery.png")), QIcon::Normal, QIcon::Off);
  tray->setIcon(icon);
}
