//
// C++ Implementation: batmonitord
//
// Description:
//
//
// Author:  Jiří Nymš<jirka@nyms.cz>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <iostream>
#include <string.h>
#include <stdlib.h>
#include <sstream>
#include <fstream>
#include <string>
#include <errno.h>
#include <dirent.h>
#include <stdio.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <pwd.h>
#include <unistd.h>
#include <list>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
//nejlepsi vec, kterou clovek muze udelat jako root, je prestat delat jako root
#define RUN_AS_USER "daemon"
#define BAT_MAX 5 //vic baterii se k notebooku snad pripojit neda
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
#define LOCK_FILE "batmonitor.lock"
#define DAEMON_NAME "batmonitord"
#define LOG_FILE "batmonitor.log"
#define PIPE_FILE ".batmonitord_pipe"
using namespace std;
typedef enum {PROC, SYS} KIFACE;

string disch_str="discharging";
int pipe_file=0;
string pipename="";

/*Odebere mezery na zacatku a konci stringu*/
void strip_string(string& s)
{
  if (s.empty())
    return;

  int a, b, begin, end;
  a=s.find_first_not_of(" ");
  b=s.find_first_not_of("\t");
  if (a>b)
    begin=a;
  else
    begin=b;

  a= s.find_last_not_of(" ");
  b= s.find_last_not_of("\t");
  if (a<b)
    end=a;
  else
    end=b;

  string temp = s;
  s.erase();
  s = temp.substr(begin, (end - begin + 1));
}

//rozdeli string na dve casti
//TODO: pridat do separatniho souboru
void split_string(const string& original, string& s1, string& s2, char splitter=':')
{
  int  position = original.find(splitter);
  s2 = original.substr(position + 1, original.size());
  s1 = original.substr(0, original.find(splitter));
}

//ten int jako reference asi neni idealni, ale v tomhle pripade nejlepsi
bool from_string(int& n, const std::string& s)
{
  istringstream iss(s);
  return !(iss >> dec >> n).fail();
}

/*Do souboru @param filename vloží @param message
neustale otvirani a zavirani je nutne, aby na XFS nedoslo k vyprazdneni souboru po vycerpani baterie
 - pokud jsem tuhle vlastnost xfs pochopil spravne
*/

void log_msg(const string& filename, const string& message)
{
  ofstream of;
  of.open(filename.c_str(), ios::app);

  if (of.is_open()) {
    of << message << endl;
    of.close();
  } else
    syslog(LOG_ERR, ("Can't open file."+filename).c_str());
}

class BatteryState
{
  protected:
    BatteryState() {}
    bool present, discharging, was_discharging; //zajima mne pouze informace, jestli se baterie vybiji
    string dir, logfile ;
    string name;
    time_t since;

    int cpu_frequency() {
      string line, s1,s2;
      ifstream in("/proc/cpuinfo");
      if (in.is_open()) {
        while (getline(in, line)) {
          split_string(line, s1, s2, ':');
          strip_string(s1);
          strip_string(s2);
          if (s1.compare("cpu MHz")==0) {
            int i;
            if (from_string(i, s2))
              in.close();
            return i;
          }
        }
        in.close();
      } else {
        syslog(LOG_ERR, "Can't read /proc/cpuinfo");
      }
      return 0;
    }


  public:
    virtual void init() {
      dir = "/proc/acpi/battery/" + name;
      ifstream in((dir+"/info").c_str());
      string line, s1, s2, msg("info;"+name+";");
      present=false;
      if (in.is_open()) {
        while (getline(in, line)) {
          split_string(line, s1, s2, ':');
          strip_string(s1);
          strip_string(s2);
          if (s1.compare("present")==0 && s2.compare("yes")==0) {
            present=true;
          } else
            if (s1.compare("design capacity")==0 || s1.compare("last full capacity")==0||s1.compare("design voltage")==0) {
              msg+=s2+";";
            }
        }
        in.close();
        if (present)
          log_msg(logfile, msg);
      }

    }

    time_t get_since() {
      return since;
    }

    BatteryState(const string& name, const string& logfile) {
      since=0;
      this->name = name;
      this->logfile=logfile;
      init();
      discharging=false;
    }


    virtual ~BatteryState() {
    }

    virtual void read(void) {   //pro stary rozhrani
      //syslog(LOG_INFO, "implemented");
      string line, msg(name + ";"), filename;
      filename = dir + "/state";
      ifstream in;
      in.open(filename.c_str());
      string s1, s2;

      if (in.is_open()) {
        was_discharging=discharging;
        discharging = false;
        while (getline(in, line)) {
          split_string(line, s1, s2, ':');
          strip_string(s1);
          strip_string(s2);
          //podminka je na hranici unosnosti delky, byt delsi, pisu to cyklem
          if (s1.compare("present rate")==0 || s1.compare("remaining capacity")==0 || s1.compare("present voltage")==0)
            msg += s2 + ";";

          if (s1.compare("charging state")==0 && s2.compare(disch_str)==0)
            discharging = true;
        }
        in.close();
      } else {
        syslog(LOG_ERR, "Can't read file %s", filename.c_str());
      }

      if (discharging) {
        if (!was_discharging)
          since=time(NULL);
        stringstream s;
        s<<cpu_frequency();
        msg+=s.str();
        log_msg(logfile, msg);
      } else {
        since=0;
      }
    }

    string get_name() const {
      return name;
    }
    bool is_present() {
      return present;
    }
};

class NewBatteryState: public BatteryState
{

    int get_value(const string& filename) {
      string line;
      int val=0;
      ifstream in(filename.c_str());
      if (in.is_open()) {
        getline(in, line);
        from_string(val, line);
        in.close();
      } else {
        syslog(LOG_ERR, "Can't read file %s", filename.c_str());
      }

      return val;
    }
  public:

    virtual void init() {
      dir = "/sys/class/power_supply/" + name;
      ifstream in((dir+"/uevent").c_str());
      string line, s1, s2;
      stringstream msg, full_design, last_full, design_voltage; //z duvodu zachovani poradi parametru
      msg << "info;" << name <<";";
      int tmp;
      present=false;
      if (in.is_open()) {
        while (getline(in, line)) {
          split_string(line, s1, s2, '=');

          if (!s1.compare("POWER_SUPPLY_PRESENT") && !s2.compare("1")) {
            present=true;
          } else
            if (!s1.compare("POWER_SUPPLY_CHARGE_FULL_DESIGN")) {
              from_string(tmp,s2);
              full_design << (int) tmp/1000 << " mAh;";
            } else
              if (!s1.compare("POWER_SUPPLY_CHARGE_FULL")) {
                from_string(tmp,s2);
                tmp=(int) tmp/1000;
                full_design << tmp << " mAh;";
              } else
                if (!s1.compare("POWER_SUPPLY_VOLTAGE_MIN_DESIGN")) {
                  from_string(tmp,s2);
                  tmp=(int) tmp/1000;
                  last_full << tmp << " mV;";
                }
        }


        msg << full_design.str() << last_full.str() << design_voltage.str();
        in.close();
        if (present)
          log_msg(logfile, msg.str());
      }


    }
    NewBatteryState(const string& name, const string& logfile) {
      this->name = name;
      this->logfile=logfile;
      init();
    }

    virtual void read(void) { //novy rozhrani

      string line, filename;
      filename = dir + "/status";
      ifstream in(filename.c_str());
      
      //present_rate, remaining_capacity, present_voltage, charging_state
      stringstream s;
      s << name << ";";
      if (in.is_open()) {
        getline(in, line);
        /*asi to bude chtit trochu poupravit, jestli esprimo bude zase ukazovat neco jinyho,
        ale zatim na nem bezi 2.6.22 a jeste nejakou dobu asi bude, takze zatim budu predpokladat,
        ze to funguje tak jak ma
        */
        if (!line.compare("Discharging")) {
          was_discharging=discharging;
          discharging=true;
          int present_rate=(int)get_value(dir+"/current_now")/1000;
          int remaining_capacity=(int) get_value(dir+"/charge_now")/1000;
          int present_voltage=(int) get_value(dir+"/voltage_now")/1000;
          s << present_rate << " mA;" << remaining_capacity << " mAh;";
          s << present_voltage << " mV;" << cpu_frequency();
          if (!was_discharging)
            since=time(NULL);

          log_msg(logfile, s.str());
        } else {
          since=0;
        }
      }

    }

    virtual ~NewBatteryState() {
    }


};

//k polozkam budu pristupovat sekvencne
list<BatteryState*> baterie = list<BatteryState*>();


/*****
Tahle třída by měla zjistit informace o baterii. Z nějakého souboru sloužícího k uložení dat
bych si měl získat něco jako poslední dosaženou kapacitu.
*/

void dealloc(list<BatteryState*>& arr)
{
  for (list<BatteryState*>::const_iterator it = arr.begin();it != arr.end();it++) {
    delete(*it);
  }
}

int getbatteries(list<BatteryState*>& batteries, KIFACE i, const string& home)
{
  const char* dir = "";
  string logfile=home+"/"+LOG_FILE;
  //syslog(LOG_INFO, logfile.c_str());
  switch (i) {

    case PROC:
      dir = "/proc/acpi/battery";
      break;

    case SYS:
      dir = "/sys/class/power_supply";
      break;
  }

  DIR *dp;
  struct dirent *dirp;

  if ((dp  = opendir(dir)) == NULL) {
    syslog(LOG_ERR, "Kernel module isn't loadedError(%d) opening %s", errno, dir);
    return errno;
  }
  BatteryState *tmp=NULL;

  while ((dirp = readdir(dp)) != NULL) {
    //odfiltrujeme aktualni a rodičovský adresář
    if (strcmp(dirp->d_name, ".") && strcmp(dirp->d_name, "..") && strcmp(dirp->d_name, "AC")) {
      switch (i) {

        case PROC:
          tmp = new BatteryState(string(dirp->d_name), logfile);
          break;

        case SYS:
          tmp = new NewBatteryState(string(dirp->d_name), logfile);
          break;
      }
      batteries.push_back(tmp);
    }
  }
  closedir(dp);
  return 0;
}

void lexit(int n)
{
  if (pipe_file!=0) {
    close(pipe_file);
    remove(pipename.c_str());
  }
  dealloc(baterie);
  exit(n);
}

static void child_handler(int signum)
{
  switch (signum) {

    case SIGUSR1:
      lexit(EXIT_SUCCESS);
      break;

    case SIGALRM:
      lexit(EXIT_FAILURE);
      break;

    case SIGTERM:
      string home= getenv("HOME");
      struct passwd *pw=getpwnam(RUN_AS_USER);;
      if (getuid() == (uid_t)pw->pw_uid || geteuid() == (uid_t) pw->pw_uid) {
        home="/var/lock";
      }
      string lockfile=home+"/"+LOCK_FILE;
      remove(lockfile.c_str());
      syslog(LOG_NOTICE, "terminated");
      closelog();
      lexit(EXIT_SUCCESS);
  }
}

/*Použit částečně modifikovaný kód z http://www-theorie.physik.unizh.ch/~dpotter/howto/daemonize - přece jen daemona píšu poprvé a nebyl jsem si jistý, co všechno je potřeba udělat.
*/
static void daemonize(const char *lockfile)
{
  pid_t pid, sid, parent;
  int lfp = -1, lofp=-1;
  const char* lf = lockfile;
  struct passwd *pw;
  //pokud uz je daemon

  if (getppid() == 1)
    return;

  /*pripadnych prav roota se zbavim co nejdriv, takze pak
  pro ukladani logu pouziju neco jineho
  */

  if (getuid() == 0 || geteuid() == 0) {

    lofp = open("/var/log/batmonitor.log", O_RDWR | O_CREAT, 0644);
    pw=getpwnam(RUN_AS_USER);;
    lf="/var/lock/batmonitor.lock";
  }
  /* Create the lock file as the current user */
  if (lf && lf[0]) {
    //odmitne se spustit, pokud existuje soubor ~/lockfile
    lfp = open(lf, O_RDWR | O_CREAT | O_EXCL, 0640);
    if (lfp < 0) {
      syslog(LOG_ERR, "unable to create lock file %s, code=%d (%s)", lf, errno, strerror(errno));
      lexit(EXIT_FAILURE);
    }
  }

  /* Drop user if there is one, and we were run as root */
  if (getuid() == 0 || geteuid() == 0) {
    if (chown(lf,(uid_t) pw->pw_uid,-1)==-1) {
      syslog(LOG_ERR, "unable to change owner of lockfile");
    }
    if (fchown(lofp,(uid_t) pw->pw_uid,-1)==-1) {
      syslog(LOG_ERR, "unable to change owner of logfile");
      close(lofp);
      lexit(EXIT_FAILURE);
    }
    if (pw) {
      syslog(LOG_NOTICE, "setting user to " RUN_AS_USER);
      setuid(pw->pw_uid);
    }
  }

  /* Trap signals that we expect to recieve */
  signal(SIGCHLD, SIG_DFL);
  signal(SIGUSR1, child_handler);
  signal(SIGALRM, child_handler);
  signal(SIGTERM, child_handler);
  signal(SIGPIPE, SIG_IGN);
  /* Fork off the parent process */
  pid = fork();

  if (pid < 0) {
    syslog(LOG_ERR, "unable to fork daemon, code=%d (%s)",
           errno, strerror(errno));
    lexit(EXIT_FAILURE);
  }

  /* If we got a good PID, then we can exit the parent process. */
  if (pid > 0) {
    /* Wait for confirmation from the child via SIGTERM or SIGCHLD, or
       for two seconds to elapse (SIGALRM).  pause() should not return. */
    alarm(2);
    pause();
    lexit(EXIT_FAILURE);
  }
  /* At this point we are executing as the child process */
  parent = getppid();
  /* Cancel certain signals */
  signal(SIGCHLD, SIG_DFL);   /* A child process dies */
  signal(SIGTSTP, SIG_IGN);   /* Various TTY signals */
  signal(SIGTTOU, SIG_IGN);
  signal(SIGTTIN, SIG_IGN);
  signal(SIGABRT, SIG_IGN);
  signal(SIGHUP, SIG_IGN);   /* Ignore hangup signal */
  /* Change the file mode mask */
  umask(0);

  /* Create a new SID for the child process */
  sid = setsid();

  if (sid < 0) {
    syslog(LOG_ERR, "unable to create a new session, code %d (%s)",
           errno, strerror(errno));
    lexit(EXIT_FAILURE);
  }

  /* record pid to lockfile */
  char str[10];
  sprintf(str, "%d\n", getpid());
  write(lfp, str, strlen(str));

  /* Change the current working directory.  This prevents the current
     directory from being locked; hence not being able to remove it. */
  if ((chdir("/")) < 0) {
    syslog(LOG_ERR, "unable to change directory to %s, code %d (%s)",
           "/", errno, strerror(errno));
    lexit(EXIT_FAILURE);
  }

  /* Redirect standard files to /dev/null */
  freopen("/dev/null", "r", stdin);
  freopen("/dev/null", "w", stdout);
  freopen("/dev/null", "w", stderr);

  /* Tell the parent process that we are A-okay */
  kill(parent, SIGUSR1);
}


int main(int argc, char *argv[])
{
  /* Initialize the logging interface */
  openlog(DAEMON_NAME, LOG_PID, LOG_LOCAL5);
  //syslog(LOG_INFO, "starting");

  string home=getenv("HOME");
  string pdir=getenv("HOME");
  if (getuid()==0) {
    home="/var/log";
    pdir="/tmp";
  }
  const char* lockfile=(home+"/"+LOCK_FILE).c_str();
  /* One may wish to process command line arguments here */
  pipename=pdir+"/" PIPE_FILE;
  if (argc > 1) {
    if (!strcmp(argv[1], "--stop")) {
      /* Root se chova trochu jinak */
      if (getuid() == 0 || geteuid() == 0) {
        struct passwd *pw = getpwnam(RUN_AS_USER);
        lockfile = "/var/lock/batmonitor.lock";
        if (pw) { //tohle by snad ani nemelo byt potreba, ale budu minimalizovat pocet ukonu jako root
          setuid(pw->pw_uid);
        }
      }
      ifstream in(lockfile);
      if (in.is_open()) {
        int i;
        string line;
        getline(in, line);
        in.close();
        cout << "trying to stop" << endl;
        cout << lockfile << endl;
        remove(lockfile);
        remove(pipename.c_str());
        if (from_string(i, line)) {
          //zkusime poslat signal ukoncujici daemona
          pid_t pid = (pid_t) i;
          kill(pid, SIGTERM);
          lexit(EXIT_SUCCESS);
        } else {
          cerr << "Nepodarilo se precist PID ze souboru." << endl;
          lexit(EXIT_FAILURE);
        }
      } else {
        cerr << "Nepodarilo se otevrit soubor" << endl;
        lexit(EXIT_FAILURE);
      }
    } else
      if (!strcmp(argv[1], "--esprimo")) {
        disch_str="charged"; //Fujitsu-Siemens Esprimo ma jeden pekny bug s prehozenym stavem baterky
      } else
        if (!strcmp(argv[1], "--new")) {
          cout << "Using new /sys interface" << endl;
        } else { //help
          cout << "batmonitor v0.0.991"<< endl;
          cout << "Usage: batmonitor [switch]" << endl;
          cout << endl;
          cout << "Parameters:" << endl;
          cout << "\t--stop\t\tStop daemon program." << endl;
          cout << "\t--help\t\tThis message." << endl;
          cout << "\t--esprimo\tSome FS Esprimo show wrong battery statuses. If you have problem with your log file, try this one" << endl;
          cout << "\t--new\t\tUse new /sys interface(since 2.6.23) - not yet implemented" << endl;
          cout << endl;
          cout << "Author: Jiri Nyms, License: GNU GPLv2 or newer" << endl;
		  return 0;
        }
  }
//  daemonize(lockfile);

  //tohle uz se provadi jako daemon
  if (argc > 1 && !strcmp(argv[1], "--new")) {
    getbatteries(baterie, SYS, home);
  } else {
    getbatteries(baterie, PROC, home);
  }
//unixový socket by byl asi lepší
  if (mkfifo(pipename.c_str(), 0660))
    syslog(LOG_ERR, "Can't create named pipe");
  pipe_file=open(pipename.c_str(), O_WRONLY | O_NONBLOCK);
  int errn=errno;
  string msg="";
  while (1) {
    stringstream ss;
    msg="";
    //notebook muze mit vice baterii
    for (list<BatteryState*>::iterator it = baterie.begin(); it != baterie.end(); it++) {
      (*it)->read();
      if ((*it)->is_present()) {
        ss << (*it)->get_name() << ":";
        ss << (*it)->get_since() << ";";
      }
    }
    ss << endl;
	
	if(pipe_file==-1 && errn==ENXIO){
	  pipe_file=open(pipename.c_str(), O_WRONLY|O_NONBLOCK);
	  if (pipe_file==-1) errn=errno;
	}
    write(pipe_file, ss.str().c_str(), strlen(ss.str().c_str()));
    sleep(30);/*30 sekund je zhruba běžná doba kdy se změní údaje
    neni to sice přesný, ale těch čísel taky nemí být moc, už takhle
    je musím odfiltrovávat*/
  }

  //konec
  close(pipe_file);
  dealloc(baterie);
  syslog(LOG_NOTICE, "terminated - program");
  closelog();
  return 0;
}
